using System;
using System.Collections.Generic;
using System.Windows.Forms;

//_DTE, Project
using EnvDTE;

//PolicyBase, PolicyFailure
using Microsoft.TeamFoundation.VersionControl.Client;

//ErrorHandler
using Microsoft.VisualStudio;

//For IVsHierarchy, IVsSolution
using Microsoft.VisualStudio.Shell.Interop;

//portions from http://www.philipwolfe.com/Post.aspx?id=91

namespace BennettAdelson.Examples
{
    [Serializable] //Class Must be Serializable
    public class CleanBuildPolicy : PolicyBase
    {
        private string icon;

        [NonSerialized]
        private _DTE m_DTE;

        private readonly string errMessage = 
            "Policy Violation (Press F1) - A clean build is required for project: {0}";

        /// <summary>
        /// Allowable Icons for Help Message from System.Windows.Forms.MessageBoxIcon
        /// </summary>
        internal static readonly List<string> IconValues = 
            new List<string>(new string[]{"None","Information","Warning","Error"});

        #region Privates

        //Code from StaticAnalysisPolicy class in the StanPolicy.dll file
        private bool ProjectUptoDate(Project project, List<PolicyFailure> failuresList)
        {
            IVsBuildableProjectCfg cfg1;
            IVsHierarchy hierarchy1 = GetVsProjectFromDTE(project);
            IVsProjectCfg2[] cfgArray1 = new IVsProjectCfg2[1];
            IVsSolutionBuildManager m_currBuildManager = PendingCheckin.GetService(typeof(IVsSolutionBuildManager)) as IVsSolutionBuildManager;

            //http://msdn2.microsoft.com/en-us/library/microsoft.visualstudio.shell.interop.ivssolutionbuildmanager.findactiveprojectcfg(VS.80).aspx
            //Determines the project configuration that is currently active. 
            m_currBuildManager.FindActiveProjectCfg(IntPtr.Zero, IntPtr.Zero, hierarchy1, cfgArray1);
            if (cfgArray1[0] == null)
            {
                return false;
            }

            //http://msdn2.microsoft.com/en-us/library/microsoft.visualstudio.shell.interop.ivsdebuggableprojectcfg.get_buildableprojectcfg(VS.80).aspx
            //Provides access to a configuration's IVsBuildableProjectCfg interface to use to manage the build process.
            cfgArray1[0].get_BuildableProjectCfg(out cfg1);
            if (cfg1 == null)
            {
                return false;
            }
            int[] numArray1 = new int[1];
            int[] numArray2 = new int[1];

            //http://msdn2.microsoft.com/en-us/library/microsoft.visualstudio.package.buildableprojectconfig.querystartuptodatecheck(VS.80).aspx
            //Determines whether or not a project is able to check if the current build is up to date. 
            int num1 = cfg1.QueryStartUpToDateCheck(1, numArray1, numArray2);
            if ((numArray1[0] != 0) && !ErrorHandler.Failed(num1))
            {
                //http://msdn2.microsoft.com/en-us/library/microsoft.visualstudio.shell.interop.ivsbuildableprojectcfg.startuptodatecheck(VS.80).aspx
                //Requests that a project begin an up-to-date check to determine whether or not the current build is up to date. 
                num1 = cfg1.StartUpToDateCheck(null, 1);
                if (ErrorHandler.Failed(num1))
                {
                    string text1 = String.Format(errMessage, project.Name);
                    failuresList.Add(new PolicyFailure(text1, this));
                    return false;
                }
            }
            return true;
        }

        private IVsHierarchy GetVsProjectFromDTE(Project project)
        {
            IVsHierarchy hierarchy1;
            string projectName = project.UniqueName;

            IVsSolution solution = PendingCheckin.GetService(typeof(IVsSolution)) as IVsSolution;

            //http://msdn2.microsoft.com/en-us/library/microsoft.visualstudio.shell.interop.ivssolution.getprojectofuniquename(VS.80).aspx
            //Returns the project in the solution, given a unique name. 
            solution.GetProjectOfUniqueName(projectName, out hierarchy1);

            return hierarchy1;
        }
        #endregion

        #region Overrides
        public override string Description
        {
            get { return "Require build before checking in"; }
        }

        public override bool Edit(IPolicyEditArgs policyEditArgs)
        {
            //Dispaly Settings UI for Custom Policy here
            using (FrmDlgSettings dlg = new FrmDlgSettings())
            {
                //Add Team Project Name to Dialog Box Title
                dlg.Text += " for " + policyEditArgs.TeamProject.Name;

                if (policyEditArgs.IsNew)
                {
                    //for new policy, don't prompt, default to Warning
                    icon = "Warning";
                    return true;
                }
                else
                {
                    //current selection for Dialog Box Default
                    dlg.SelectedIcon = icon;
                    DialogResult dr = dlg.ShowDialog(policyEditArgs.Parent);//NOTE: Use Parent for window positioning
                    if (dr == DialogResult.OK)
                    {
                        //Save User Selection
                        icon = dlg.SelectedIcon;
                        return true;
                    }
                    else
                    {
                        //Dialog Box Canceled
                        return false;
                    }
                }
            }
        }

        /// <summary>
        /// Evaluates the Custom Check-In Policy
        /// </summary>
        /// <returns>A List of Policy Failures</returns>
        /// <remarks>List is Empty if no policy failures occurred.</remarks>
        public override PolicyFailure[] Evaluate()
        {
            try
            {
                List<PolicyFailure> failureList = new List<PolicyFailure>();
               
                m_DTE = PendingCheckin.GetService(typeof(_DTE)) as _DTE;

                if (m_DTE != null)
                {
                    if (m_DTE.Solution != null)
                    {
                        if (m_DTE.Solution.Projects != null)
                        {
                            foreach (Project project in m_DTE.Solution.Projects)
                            {
                                ProjectUptoDate(project, failureList);
                            }
                        }
                    }
                }

                return failureList.ToArray();

            }
            catch
            {
                return new PolicyFailure[0]; // on error, don't fail policy, ignore
            }
        }

        public override string Type
        {
            get { return "Clean Build Policy"; }
        }

        public override string TypeDescription
        {
            get { return "This policy will prompt the user to build without errors before check in."; }
        }

        public override string InstallationInstructions
        {
            get
            {
                return "To install this policy, register this assembly in the Checkin Policies folder under TeamFoundation in the registry.";
            }
         }

        public override void DisplayHelp(PolicyFailure failure)
        {
            MessageBox.Show(
                                "Please perform a clean build before you check in your code.", 
                                "Clean Build Policy",
                                MessageBoxButtons.OK,
                                (MessageBoxIcon)Enum.Parse(typeof(MessageBoxIcon),icon) //Example Custom Setting from Dialog Box selection
                            );
        }
        #endregion
    }
}
