using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Resources;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace BennettAdelson.Samples.FXCopRuleExplorer
{
    public partial class FrmMain : Form
    {
        private static int failedSearchCount;

        private const string allRules = "*ALL*";

        private static readonly string rulePath =
                Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + 
                @"\Microsoft Visual Studio 8\Team Tools\Static Analysis Tools\FxCop\Rules";

        private Dictionary<string, XmlDocument> ruleLists = new Dictionary<string, XmlDocument>();

        public FrmMain()
        {
            InitializeComponent();
            
            fillFileList();
        }

        private void fillFileList()
        {
            lstFiles.Items.Clear();

            lstFiles.Items.Add(allRules);
            
            foreach (string fileName in Directory.GetFiles(rulePath, "*.dll"))
            {
                lstFiles.Items.Add(Path.GetFileNameWithoutExtension(fileName));
            }
        }

        private XmlDocument getRuleListForEveryAssembly()
        {
            if (!ruleLists.ContainsKey(allRules))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("<AllRules>");
                foreach (object listItem in lstFiles.Items)
                {
                    string asmName = listItem.ToString();
                    if (asmName.Equals(allRules)) continue; //skip *ALL*
                    sb.Append(getRuleListForAssembly(asmName).OuterXml);
                }
                sb.AppendLine("</AllRules>");

                XmlDocument xd = new XmlDocument();
                xd.LoadXml(sb.ToString());
                ruleLists.Add(allRules, xd);
            }

            return ruleLists[allRules];
        }

        private XmlDocument getRuleListForAssembly(string assemblyName)
        {
            if (!ruleLists.ContainsKey(assemblyName))
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(readXMLResourceFromAssembly(assemblyName));
                ruleLists.Add(assemblyName, doc);
            }

            return ruleLists[assemblyName];
        }

        private static string readXMLResourceFromAssembly(string assemblyName)
        {
            string asmFile = Path.Combine(rulePath, assemblyName + ".dll");
            Assembly asm = Assembly.LoadFile(asmFile);

            //default for debugging assemblies with no .xml Resources found
            string content = "<assemblyName>" + assemblyName + "</assemblyName>";

            foreach (string name in asm.GetManifestResourceNames())
            {
                if (name.ToString().EndsWith(".xml",StringComparison.CurrentCultureIgnoreCase))
                {
                    //MessageBox.Show("Found: " + name);
                    using (StreamReader sr = new StreamReader(asm.GetManifestResourceStream(name)))
                    {
                        content = sr.ReadToEnd();
                    }

                    break; //stop looking for resources (only one per assembly)
                }
            }

            return content;
        }

        private void displayXML(string xmlToDisplay)
        {
            string fn = "disp.xml";
            fn = Path.Combine(Environment.CurrentDirectory, fn);
            File.WriteAllText(fn, xmlToDisplay);
            wbRuleDisplay.Navigate(fn);
        }

        private void lstFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            XmlDocument ruleDoc;

            switch (lstFiles.SelectedIndex)
            {
                case -1:
                    //No Selection
                    return; 
                case 0:
                    //ALL Rules
                    ruleDoc = getRuleListForEveryAssembly();
                    break;
                default:
                    //Individual Rule Assembly
                    ruleDoc = getRuleListForAssembly(lstFiles.SelectedItem.ToString());
                    break;
            }

            pg.SelectedObject = new RuleXMLUtilities(ruleDoc);
            
            if (txtNameSearch.Text.Trim().Length > 3)
                filterListBySearch();
            else
                displayXML(ruleDoc.OuterXml);
        }

        private void txtNameSearch_TextChanged(object sender, EventArgs e)
        {
            txtNameSearch.ReadOnly = true;
            filterListBySearch();
            txtNameSearch.ReadOnly = false;
        }

        private void filterListBySearch()
        {
            RuleXMLUtilities util = pg.SelectedObject as RuleXMLUtilities;
            if (util != null && txtNameSearch.Text.Trim().Length > 3)
            {
                XmlDocument xd = util.FilterByRuleName(txtNameSearch.Text.Trim());
                if (xd != null)
                {
                    displayXML(xd.OuterXml);
                    if (util.NumFound == 0) failedSearchCount++;

                    if (failedSearchCount > 5)
                    {
                        System.Threading.Thread.Sleep(500);
                        failedSearchCount = 0;
                    }
                }
            }
            else if (txtNameSearch.Text.Trim().Length == 0)
            {
                //Clear Search
                if (util.NumFound == 0) return; //already cleared

                failedSearchCount = 0;

                displayXML(util.GetCurrentDocument().OuterXml);
                util.NumFound = 0;
            }

            pg.Refresh();
        }
    }
}